@@ -90,6 +90,8 @@ static CONFIG config[]=
 { "control",   1,1,1 },
 { "packetize", 1,0,0 },
 { "smallrtp",  1,0,0 },
+{ "recode",    1,1,1 },
+{ "tun",       1,0,0 },
 { NULL,        0,0,0 }
 };
 
@@ -138,6 +140,8 @@ static CONFIG config[]=
 #define IDX_CONTROL		49
 #define IDX_PACKETIZE		50
 #define IDX_SMALLRTP		51
+#define IDX_RECODE		52
+#define IDX_TUN			53
 
 #define CFG_MODE(a)		anchor->config[IDX_MODE].a
 #define CFG_LOCAL(a)		anchor->config[IDX_LOCAL].a
@@ -256,6 +260,8 @@ static NUMDATA numdata[]=
 	{ IDX_PPP,        -1, 1,               0               },
 	{ IDX_PACKETIZE,  -1, 1,               0               },
 	{ IDX_SMALLRTP,   -1, 1,               0               },
+	{ IDX_RECODE,      0, 2,               0               },
+	{ IDX_TUN,        -1, 1,               0               },
 	{ 0,               0, 0,               0               }
 
 };
@@ -769,6 +775,20 @@ int parse(VPN *anchor)
 	}
 	anchor->pktmode=*u.w16++;
 	anchor->smallrtp=*u.w16++;
+	anchor->recode=*u.w16++;
+	if(*u.w16++)
+	{
+		if(anchor->ifhandler==ppphandler)
+		{
+			printf("ppp and tun given.\n");
+			return 0;
+		}
+		anchor->ifhandler=tunhandler;
+		anchor->iftype="tun";
+		anchor->pkt_init=pkg_tun_init;
+		anchor->if_to_pkt=tun_to_pkg;
+		anchor->pkt_to_if=pkg_to_tun;
+	}
 
 	/* check control socket, if given */
 
