@@ -414,19 +414,19 @@ int ppphandler(int smode,VPN *anchor)
 #if defined(SunOS)
 	ifr.ifr_addr.sa_family=AF_INET;
 	((struct sockaddr_in *)(&(ifr.ifr_addr)))->sin_addr.s_addr=
-		getaddr(anchor->local_ip);
+		getaddr((unsigned char *)(anchor->local_ip));
 	if(!j)if(ioctl(i,SIOCSIFADDR,&ifr)<0)j++;
 #elif defined(LINUX)
 	((struct sockaddr_in *)(&(ifr.ifr_addr)))->sin_family=AF_INET;
 	((struct sockaddr_in *)(&(ifr.ifr_addr)))->sin_addr.s_addr=
-		getaddr(anchor->local_ip);
+		getaddr((unsigned char *)(anchor->local_ip));
 	if(!j)if(ioctl(i,SIOCSIFADDR,&ifr)<0)j++;
 #elif defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD)
 	((struct sockaddr_in *)(&(ifa.ifra_addr)))->sin_len=
 		sizeof(struct sockaddr_in);
 	((struct sockaddr_in *)(&(ifa.ifra_addr)))->sin_family=AF_INET;
 	((struct sockaddr_in *)(&(ifa.ifra_addr)))->sin_addr.s_addr=
-		getaddr(anchor->local_ip);
+		getaddr((unsigned char *)(anchor->local_ip));
 #endif
 
 	/* set up interface peer address, react to errors */
@@ -434,19 +434,19 @@ int ppphandler(int smode,VPN *anchor)
 #if defined(SunOS)
 	ifr.ifr_addr.sa_family=AF_INET;
 	((struct sockaddr_in *)(&(ifr.ifr_addr)))->sin_addr.s_addr=
-		getaddr(anchor->remote_ip);
+		getaddr((unsigned char *)(anchor->remote_ip));
 	if(!j)if(ioctl(i,SIOCSIFDSTADDR,&ifr)<0)j++;
 #elif defined(LINUX)
 	((struct sockaddr_in *)(&(ifr.ifr_dstaddr)))->sin_family=AF_INET;
 	((struct sockaddr_in *)(&(ifr.ifr_dstaddr)))->sin_addr.s_addr=
-		getaddr(anchor->remote_ip);
+		getaddr((unsigned char *)(anchor->remote_ip));
 	if(!j)if(ioctl(i,SIOCSIFDSTADDR,&ifr)<0)j++;
 #elif defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD)
 	((struct sockaddr_in *)(&(ifa.ifra_broadaddr)))->sin_len=
 		sizeof(struct sockaddr_in);
 	((struct sockaddr_in *)(&(ifa.ifra_broadaddr)))->sin_family=AF_INET;
 	((struct sockaddr_in *)(&(ifa.ifra_broadaddr)))->sin_addr.s_addr=
-		getaddr(anchor->remote_ip);
+		getaddr((unsigned char *)(anchor->remote_ip));
 #endif
 
 	/* set up interface netmask, react to errors */
@@ -527,8 +527,9 @@ int ppphandler(int smode,VPN *anchor)
 #ifdef LINUX
 	if(!anchor->autoroute)
 	{
-		if(route(1,getaddr(anchor->remote_ip),-1,0,ifr.ifr_name))
-			GOTO("autoroute",err5);
+		if(route(1,getaddr((unsigned char *)(anchor->remote_ip)),-1,0,
+			(unsigned char *)(ifr.ifr_name)))
+				GOTO("autoroute",err5);
 	}
 #endif
 
@@ -536,8 +537,9 @@ int ppphandler(int smode,VPN *anchor)
 
 	if(anchor->peerroute)
 		if(route(1,anchor->peerip,-1,
-			anchor->peerroute>1?anchor->peergate:0,anchor->peerdev))
-		GOTO("peerroute",err6);
+			anchor->peerroute>1?anchor->peergate:0,
+			(unsigned char *)(anchor->peerdev)))
+				GOTO("peerroute",err6);
 
 	/* set up all additional routes */
 
@@ -603,7 +605,8 @@ err7:	if(quiet)err++;
 	if(anchor->peerroute)
 		if(route(0,anchor->peerip,-1,
 			anchor->peerroute>1?anchor->peergate:0,
-			anchor->peerdev))if(!quiet++)DEBUG1(3,"peerroute");
+			(unsigned char *)(anchor->peerdev)))
+				if(!quiet++)DEBUG1(3,"peerroute");
 
 err6:	if(quiet)err++;
 
@@ -612,8 +615,9 @@ err6:	if(quiet)err++;
 #ifdef LINUX
 	if(!anchor->autoroute)
 	{
-		if(route(0,getaddr(anchor->remote_ip),-1,0,ifr.ifr_name))
-			if(!quiet++)DEBUG1(3,"autoroute");
+		if(route(0,getaddr((unsigned char *)(anchor->remote_ip)),-1,0,
+			(unsigned char *)(ifr.ifr_name)))
+				if(!quiet++)DEBUG1(3,"autoroute");
 	}
 
 err5:	if(quiet)err++;
