@@ -307,7 +307,8 @@ int main(int argc,char *argv[])
 
 		/* create key schedule from password */
 
-		blowfish_key_schedule(pwd,strlen(pwd),globals.u3.sched);
+		blowfish_key_schedule((unsigned char *)pwd,strlen(pwd),
+			globals.u3.sched);
 
 		/* overwrite password memory */
 
