@@ -56,6 +56,7 @@
 #endif
 #include <net/ppp_defs.h>
 #include <linux/if_ppp.h>
+#include <linux/if_tun.h>
 #endif
 #include <sys/resource.h>
 #if defined(FreeBSD) || defined(OSNetBSD) || defined(OSOpenBSD)
@@ -66,6 +67,7 @@
 #endif
 #endif
 #ifdef FreeBSD
+#include <net/if_tun.h>
 #include <net/if_var.h>
 #endif
 #ifdef SunOS
@@ -128,7 +130,7 @@
 
 /* special definitions, do not change!!! */
 
-#define VERSION "1.1.3"	/* current vpnd version number			  */
+#define VERSION "1.1.4"	/* current vpnd version number			  */
 #define MAXBLOCK 2048	/* largest block size transferred, do not change! */
 #define MAXIDLE 900	/* longest link idle time before link test	  */
 #define XON 0x11	/* XON (for escaping/filtering)			  */
@@ -228,6 +230,18 @@
 #define CTRL_ISUP	2		/* peer link is up		*/
 #define CTRL_ISDOWN	3		/* peer link is down		*/
 
+/* icmp compression, do not change! */
+
+#define ICMPMAX		32		/* icmp compression slots	*/
+
+/* udp compression, do not change! */
+
+#define UDPMAX		32		/* udp compression slots	*/
+
+/* tcp compression, do not change! */
+
+#define TCPMAX		16		/* tcp compression slots	*/
+
 /* rtp compression, do not change! */
 
 #define RTPMAX		32		/* rtp compression slots	*/
@@ -241,9 +255,115 @@
 #define VJC_UNCOMP	3		/* Van Jacobsen uncompressed TCP*/
 #define RTP_COMP	4		/* RFC2508 compressed RTP	*/
 #define RTP_UNCOMP	5		/* RFC2508 uncompressed RTP	*/
+#define ICMPUDP_COMP	6		/* compressed ICMP/UDP		*/
+#define ICMPUDP_UNCOMP	7		/* uncompressed ICMP/UDP	*/
 #define STREAM		16		/* converted packet is ready	*/
 #define ERR		17		/* packet is in error state	*/
 
+/* ip header */
+
+typedef struct
+{
+WORD08 verihl;				/* ip version/header length	*/
+WORD08 tos;				/* ip type of service		*/
+WORD16 tot_len;				/* ip total datagram length	*/
+WORD16 id;				/* ip identification		*/
+WORD16 frag_off;			/* ip flags/fragmentation offset*/
+WORD08 ttl;				/* ip time to live		*/
+WORD08 protocol;			/* ip protocol			*/
+WORD16 check;				/* ip header checksum		*/
+WORD32 saddr;				/* ip source address		*/
+WORD32 daddr;				/* ip destination address	*/
+WORD32 options[0];			/* ip options and padding	*/
+} IPH;
+
+/* icmp header */
+
+typedef struct
+{
+WORD16 typecode;			/* icmp type and code		*/
+WORD16 chksum;				/* icmp datagram checksum	*/
+} ICMP;
+
+/* udp header */
+
+typedef struct
+{
+WORD16 source;				/* udp source port		*/
+WORD16 dest;				/* udp destination port		*/
+WORD16 len;				/* udp datagram length		*/
+WORD16 chksum;				/* udp datagram checksum	*/
+} UDP;
+
+/* tcp header */
+
+typedef struct
+{
+WORD16 sport;				/* tcp source port		*/
+WORD16 dport;				/* tcp destination port		*/
+WORD32 seqnr;				/* tcp sequence number		*/
+WORD32 acknr;				/* tcp acknowledge number	*/
+WORD16 do_ecn_cb;			/* tcp data offset, ecn and cb	*/
+WORD16 window;				/* tcp window			*/
+WORD16 chksum;				/* tcp checksum			*/
+WORD16 urgp;				/* tcp urgent pointer		*/
+WORD32 options[0];			/* tcp options and padding	*/
+} TCP;
+
+/* icmp compression state work area */
+
+typedef struct _icmpstate
+{
+struct _icmpstate *next;		/* next work area in ring	*/
+union
+{
+  WORD08 data[60];			/* maximum ip header length	*/
+  IPH ip;				/* ip header			*/
+} ip;					/* ip header storage area	*/
+union
+{
+  WORD08 data[4];			/* maximum tcp header length	*/
+  ICMP icmp;				/* icmp header			*/
+} icmp;					/* icmp header storage area	*/
+WORD08 id;				/* work area id			*/
+} ICMPSTATE;
+
+/* udp compression state work area */
+
+typedef struct _udpstate
+{
+struct _udpstate *next;			/* next work area in ring	*/
+union
+{
+  WORD08 data[60];			/* maximum ip header length	*/
+  IPH ip;				/* ip header			*/
+} ip;					/* ip header storage area	*/
+union
+{
+  WORD08 data[8];			/* maximum tcp header length	*/
+  UDP udp;				/* udp header			*/
+} udp;					/* udp header storage area	*/
+WORD08 id;				/* work area id			*/
+} UDPSTATE;
+
+/* tcp compression state work area */
+
+typedef struct _tcpstate
+{
+struct _tcpstate *next;			/* next work area in ring	*/
+union
+{
+  WORD08 data[60];			/* maximum ip header length	*/
+  IPH ip;				/* ip header			*/
+} ip;					/* ip header storage area	*/
+union
+{
+  WORD08 data[60];			/* maximum tcp header length	*/
+  TCP tcp;				/* tcp header			*/
+} tcp;					/* tcp header storage area	*/
+WORD08 id;				/* work area id			*/
+} TCPSTATE;
+
 /* ip/udp/rtp header */
 
 typedef struct
@@ -275,6 +395,7 @@ WORD32 csrc[15];			/* rtp contributing source	*/
 typedef struct _rtpstate
 {
 struct _rtpstate *next;			/* next work area in ring	*/
+void *recode;				/* recode stream handle		*/
 WORD08 id;				/* work area id			*/
 WORD08 seq;				/* compression sequence number	*/
 WORD16 ipid;				/* current ip id delta		*/
@@ -303,14 +424,30 @@ int state;				/* current processing state	*/
 int length;				/* output data total length	*/
 int total;				/* input bytes total amount	*/
 int processed;				/* input bytes processed amount	*/
+int vj;					/* tcp header compression flag	*/
+WORD08 icmplastxmit;			/* last icmp compression index	*/
+WORD08 icmplastrecv;			/* last icmp compression index	*/
+WORD08 udplastxmit;			/* last udp compression index	*/
+WORD08 udplastrecv;			/* last udp compression index	*/
+WORD08 tcplastxmit;			/* last tcp compression index	*/
+WORD08 tcplastrecv;			/* last tcp compression index	*/
 WORD08 *data;				/* pointer to actual output data*/
-WORD16 chksum;				/* ppp checksum generation/vry	*/
-WORD08 bfr[(2*MAXBLOCK)+9];		/* work buffer (output)		*/
 RTPSTATE *rtplast;			/* current compression state	*/
-RTPSTATE rtpstate[RTPMAX];		/* rtp compression state array	*/
 NEGSTATE *neglast;			/* current negative cache state	*/
+ICMPSTATE *icmplastx;			/* last icmp compression slot	*/
+UDPSTATE *udplastx;			/* last udp compression slot	*/
+TCPSTATE *tcplastx;			/* last tcp compression slot	*/
+RTPSTATE rtpstate[RTPMAX];		/* rtp compression state array	*/
 NEGSTATE negstate[RTPNEG];		/* negative compression cache	*/
-WORD08 hdr[198];			/* compression work buffer	*/
+ICMPSTATE icmpstatex[ICMPMAX];		/* sending icmp compression st. */
+ICMPSTATE icmpstater[ICMPMAX];		/* recving icmp compression st. */
+UDPSTATE udpstatex[UDPMAX];		/* sending udp compression state*/
+UDPSTATE udpstater[UDPMAX];		/* recving udp compression state*/
+TCPSTATE tcpstatex[TCPMAX];		/* sending tcp compression state*/
+TCPSTATE tcpstater[TCPMAX];		/* recving tcp compression state*/
+WORD16 chksum;				/* ppp checksum generation/vry	*/
+WORD08 hdr[358];			/* compression work buffer	*/
+WORD08 bfr[(2*MAXBLOCK)+9];		/* work buffer (output)		*/
 } PKG;
 
 /* configuration file parse result */
@@ -453,10 +590,11 @@ int mux;				/* Solaris ppp setup (data mux)	*/
 int (*ifhandler)(int,struct _vpn  *);	/* interface handler to use	*/
 char *iftype;				/* name of interface type	*/
 void (*pkt_init)(PKG *p);		/* packet init routine to use	*/
-int (*if_to_pkt)(PKG *p,WORD08 *i, int l);/* if to packet routine to use*/
-int (*pkt_to_if)(PKG *p,WORD08 *i, int l);/* packet to if routine to use*/
+int (*if_to_pkt)(PKG *p,WORD08 *i, int l,int r);/* if to packet routine */
+int (*pkt_to_if)(PKG *p,WORD08 *i, int l,int r);/* packet to if routine */
 int pktmode;				/* pkg operation mode flag	*/
 int smallrtp;				/* minimize rtp packets flag	*/
+int recode;				/* recode G.711 to speex	*/
 PKG ifpkt;				/* if to packet work area	*/
 PKG pktif;				/* packet to if work area	*/
 struct _vpn *next;			/* next set of global data	*/
@@ -488,6 +626,10 @@ extern int sliphandler(int,VPN *);
 
 extern int ppphandler(int,VPN *);
 
+/* function prototypes for tun.c */
+
+extern int tunhandler(int,VPN *);
+
 /* function prototypes for serial.c */
 
 extern int serialhandler(VPN *);
@@ -587,10 +729,21 @@ extern int docontrol(int);
 
 /* function prototypes for packetizer.c */
 
-extern void rtpcomp_init(PKG *dta);
+extern void pkgcomp_init(PKG *dta,int vj);
+extern void pkgcomp_exit(PKG *dta);
 extern void pkg_slip_init(PKG *dta);
-extern int slip_to_pkg(PKG *dta,WORD08 *in,int ilen);
-extern int pkg_to_slip(PKG *dta,WORD08 *in,int ilen);
+extern int slip_to_pkg(PKG *dta,WORD08 *in,int ilen,int recode);
+extern int pkg_to_slip(PKG *dta,WORD08 *in,int ilen,int recode);
 extern void pkg_ppp_init(PKG *dta);
-extern int ppp_to_pkg(PKG *dta,WORD08 *in,int ilen);
-extern int pkg_to_ppp(PKG *dta,WORD08 *in,int ilen);
+extern int ppp_to_pkg(PKG *dta,WORD08 *in,int ilen,int recode);
+extern int pkg_to_ppp(PKG *dta,WORD08 *in,int ilen,int recode);
+extern void pkg_tun_init(PKG *dta);
+extern int tun_to_pkg(PKG *dta,WORD08 *in,int ilen,int recode);
+extern int pkg_to_tun(PKG *dta,WORD08 *in,int ilen,int recode);
+
+/* function prototypes for recode.c */
+
+extern void *recode_init(int mode,void *old);
+extern void *recode_exit(void *old);
+extern int recode_alaw(void *handle,char *data,int len,int max);
+extern int recode_ulaw(void *handle,char *data,int len,int max);
